BackoffUntilCondition

Trait BackoffUntilCondition 

pub trait BackoffUntilCondition {
    type Result;

    // Required method
    fn into_result(self) -> Option<Self::Result>;
}
A condition checked in BackoffStrategy::backoff_until.

It should typically be a bool or an Option<T>.

Required Associated Types

type Result

The result to return when the condition is satisfied.

Required Methods

fn into_result(self) -> Option<Self::Result>

Converts the condition into a result to be returned.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementations on Foreign Types

Source

impl BackoffUntilCondition for bool

Source

impl<T> BackoffUntilCondition for Option<T>

Source

type Result = T

Source

fn into_result(self) -> Option<Self::Result>

Implementors