pub trait BackoffUntilCondition {
type Result;
// Required method
fn into_result(self) -> Option<Self::Result>;
}Expand description
A condition checked in BackoffStrategy::backoff_until.
It should typically be a bool or an Option<T>.
Required Associated Types
Required Methods
Sourcefn into_result(self) -> Option<Self::Result>
fn into_result(self) -> Option<Self::Result>
Converts the condition into a result to be returned.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".